@@ -27,6 +27,8 @@ from json2html import Json2Html
 from IPython.display import display, HTML
 from boltons.iterutils import remap
 from pymatgen import Structure
+from concurrent.futures import as_completed
+from requests_futures.sessions import FuturesSession
 
 
 DEFAULT_HOST = "api.mpcontribs.org"
@@ -140,6 +142,8 @@ class Client(SwaggerClient):
         self.apikey = apikey
         self.headers = {"x-api-key": apikey} if apikey else headers
         self.host = host
+        self.protocol = "https" if self.apikey else "http"
+        self.url = f"{self.protocol}://{self.host}"
 
         if "swagger_spec" not in self.__dict__ or (
             self.headers is not None
@@ -150,11 +154,10 @@ class Client(SwaggerClient):
     def load(self):
         http_client = FidoClientGlobalHeaders(headers=self.headers)
         loader = Loader(http_client)
-        protocol = "https" if self.apikey else "http"
-        origin_url = f"{protocol}://{self.host}/apispec.json"
+        origin_url = f"{self.url}/apispec.json"
         spec_dict = loader.load_spec(origin_url)
         spec_dict["host"] = self.host
-        spec_dict["schemes"] = [protocol]
+        spec_dict["schemes"] = [self.protocol]
 
         config = {
             "validate_responses": False,
@@ -260,30 +263,68 @@ class Client(SwaggerClient):
             ).result()
         )
 
-    def delete_contributions(self, project, limit=200):
+    def delete_contributions(self, project, per_page=100):
         """Convenience function to remove all contributions for a project"""
         resp = self.contributions.get_entries(
-            project=project, _fields=["id"], _limit=1
+            project=project, per_page=per_page, _fields=["id"]
         ).result()
+        cids = [d["id"] for d in resp["data"]]  # in first page
         ncontribs = resp["total_count"]
 
-        if ncontribs:
-            has_more = True
-
+        if cids:
             with tqdm(total=ncontribs) as pbar:
-                pbar.set_description("Delete contribution(s)")
-                while has_more:
-                    resp = self.contributions.delete_entries(
-                        project=project, _limit=limit
-                    ).result()
-                    has_more = resp["has_more"]
-                    pbar.update(resp["count"])
+                with FuturesSession(max_workers=10) as session:
+                    # bravado future unfortunately doesn't work with concurrent.futures
+                    pbar.set_description("Get contribution IDs")
+                    endpoint = f"{self.url}/contributions/"
+                    pages = resp["total_pages"]
+                    futures = [
+                        session.get(
+                            endpoint,
+                            headers=self.headers,
+                            params={
+                                "project": project,
+                                "_fields": "id",
+                                "page": page,
+                                "per_page": per_page,
+                            },
+                        )
+                        for page in range(2, pages + 1)
+                    ]
 
-                if resp["count"]:
-                    self.load()
+                    for future in as_completed(futures):
+                        resp = future.result().json()
+                        cids += [d["id"] for d in resp["data"]]
+                        pbar.update(len(resp["data"]))
+
+                    pbar.refresh()
+                    pbar.set_description("Delete contribution(s)")
+                    futures = [
+                        session.delete(
+                            endpoint,
+                            headers=self.headers,
+                            params={
+                                "project": project,
+                                "id__in": ",".join(chunk),
+                                "per_page": per_page,
+                            },
+                        )
+                        for chunk in chunks(cids, n=per_page)
+                    ]
+
+                    resp = None
+                    for future in as_completed(futures):
+                        resp = future.result().json()
+                        if "count" in resp:
+                            pbar.update(resp["count"])
+                        elif "error" in resp:
+                            warnings.warn(resp["error"])
+
+                    if resp and resp.get("count"):
+                        self.load()
 
     def submit_contributions(
-        self, contributions, skip_dupe_check=False, ignore_dupes=False, limit=200
+        self, contributions, skip_dupe_check=False, ignore_dupes=False, per_page=100
     ):
         """Convenience function to submit a list of contributions"""
         # prepare structures/tables
@@ -295,6 +336,7 @@ class Client(SwaggerClient):
                 "tables": set(),
             }
             unique_identifiers = True
+            endpoint = f"{self.url}/contributions/"
 
             if not skip_dupe_check:
                 name = contributions[0]["project"]
@@ -304,27 +346,39 @@ class Client(SwaggerClient):
                 unique_identifiers = resp["unique_identifiers"]
 
                 pbar.set_description("Get existing contribution(s)")
-                has_more, page, per_page = True, 1, 250
+                resp = self.contributions.get_entries(
+                    project=name, per_page=per_page, _fields=["id"],
+                ).result()
+                pages = resp["total_pages"]
 
-                while has_more:
-                    resp = self.contributions.get_entries(
-                        project=name,
-                        page=page,
-                        per_page=per_page,
-                        _fields=["id", "identifier", "structures", "tables"],
-                    ).result()
+                with FuturesSession(max_workers=10) as session:
+                    # bravado future unfortunately doesn't work with concurrent.futures
+                    futures = [
+                        session.get(
+                            endpoint,
+                            headers=self.headers,
+                            params={
+                                "project": name,
+                                "page": page + 1,
+                                "per_page": per_page,
+                                "_fields": "id,identifier,structures,tables",
+                            },
+                        )
+                        for page in range(pages)
+                    ]
 
-                    for contrib in resp["data"]:
-                        existing["ids"].add(contrib["id"])
-                        existing["identifiers"].add(contrib["identifier"])
+                    for future in as_completed(futures):
+                        resp = future.result().json()
 
-                        for component in ["structures", "tables"]:
-                            md5s = set(d["md5"] for d in contrib[component])
-                            existing[component] |= md5s
+                        for contrib in resp["data"]:
+                            existing["ids"].add(contrib["id"])
+                            existing["identifiers"].add(contrib["identifier"])
 
-                    has_more = resp["has_more"]
-                    pbar.update(page * per_page)
-                    page += 1
+                            for component in ["structures", "tables"]:
+                                md5s = set(d["md5"] for d in contrib[component])
+                                existing[component] |= md5s
+
+                            pbar.update(1)
 
                 nexisting = 0
                 if unique_identifiers and existing["identifiers"]:
@@ -409,12 +463,27 @@ class Client(SwaggerClient):
                 pbar.refresh()
                 pbar.reset(total=ncontribs)
                 pbar.set_description("Submit contribution(s)")
+                resp = None
 
-                for chunk in chunks(contribs, n=limit):
-                    resp = self.contributions.create_entries(
-                        contributions=chunk
-                    ).result()
-                    pbar.update(resp["count"])
+                with FuturesSession(max_workers=10) as session:
+                    # bravado future unfortunately doesn't work with concurrent.futures
+                    headers = {"Content-Type": "application/json"}
+                    headers.update(self.headers)
+                    futures = [
+                        session.post(
+                            endpoint,
+                            headers=headers,
+                            data=json.dumps(chunk).encode("utf-8"),
+                        )
+                        for chunk in chunks(contribs, n=per_page)
+                    ]
 
-                if resp["count"]:
-                    self.load()
+                    for future in as_completed(futures):
+                        resp = future.result().json()
+                        if "count" in resp:
+                            pbar.update(resp["count"])
+                        elif "error" in resp:
+                            warnings.warn(resp["error"])
+
+                    if resp and resp.get("count"):
+                        self.load()
